@@ -27,6 +27,10 @@
 --
 --
 -- $Log: compress_sparse.c,v $
+-- Revision 1.22  2008/03/12 03:56:38  jsedwards
+-- Changed to use the GNU MD5 context and functions instead of the RSA context
+-- and functions.
+--
 -- Revision 1.21  2008/02/03 01:15:40  jsedwards
 -- Change to use nwos_get_private_objects_path function instead of DEFAULT_FILE.
 --
@@ -136,9 +140,10 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#include "md5/md5.h"
+#include "gnu/md5.h"
+#include "gnu/sha1.h"
+
 #include "objectify_private.h"
-#include "sha1/sha1.h"
 
 
 static void print_usage(char *program)
@@ -169,7 +174,7 @@ int main(int argc, char* argv[])
     Disk_Header disk_header;
     uint32 counts[SIZE_COUNTS];
     uint32 ref;
-    MD5_CTX md5_context;    /* MD5 checksum context */
+    struct md5_ctx md5_context;    /* MD5 checksum context */
     struct sha1_ctx sha1_context;
     uint8 md5_digest[16];
     uint8 sha1_digest[20];
@@ -274,7 +279,7 @@ int main(int argc, char* argv[])
 	}
     }
 
-    MD5Init(&md5_context);   /* initialize the MD5 checksum context */
+    md5_init_ctx(&md5_context);   /* initialize the MD5 checksum context */
     sha1_init_ctx(&sha1_context);
 
     printf("header: %c%c%c%c %c%c%c%c\n",
@@ -292,7 +297,7 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
-    MD5Update(&md5_context, block, (unsigned)sizeof(block));    /* include this data in the md5 checksum */
+    md5_process_bytes(block, sizeof(block), &md5_context);    /* include this data in the md5 checksum */
     sha1_process_bytes(block, sizeof(block), &sha1_context);    /* include this data in the sha1 checksum */
 
     num_blocks = 0;
@@ -354,7 +359,7 @@ int main(int argc, char* argv[])
 		    exit(1);
 		}
 
-		MD5Update(&md5_context, block, (unsigned)sizeof(block));    /* include this data in the md5 checksum */
+		md5_process_bytes(block, sizeof(block), &md5_context);    /* include this data in the md5 checksum */
 		sha1_process_bytes(block, sizeof(block), &sha1_context);    /* include this data in the sha1 checksum */
 		num_blocks++;
 	    }
@@ -363,7 +368,7 @@ int main(int argc, char* argv[])
 
     printf("Number of blocks: %d\n", num_blocks);
 
-    MD5Final(md5_digest, &md5_context);   /* finish computing the md5 sum */
+    md5_finish_ctx(&md5_context, md5_digest);   /* finish computing the md5 sum */
     sha1_finish_ctx(&sha1_context, sha1_digest);
 
     printf("MD5: ");
