@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                Copyright (C) 2006, 2007  QRW Software
+--                 Copyright (C) 2006-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,15 @@
 --
 --
 -- $Log: disc_list.c,v $
+-- Revision 1.18  2008/02/03 01:06:30  jsedwards
+-- Changed DEFAULT_TYPE_RW to READ_WRITE.
+--
+-- Revision 1.17  2007/12/24 02:20:53  jsedwards
+-- Added --add-revision option to connect any new files with any existing files.
+--
+-- Revision 1.16  2007/12/23 18:23:25  jsedwards
+-- Added terminate_objectify call before exiting in disc already exists case.
+--
 -- Revision 1.15  2007/10/07 03:43:50  jsedwards
 -- Renamed 'nwos_set_block_estimate' to 'nwos_check_blocks_available' and
 -- changed to exit gracefully if not enough space available.
@@ -104,9 +113,82 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include "objectify.h"
+#include "objectify_private.h"     /* this is kinda bad, but need overwrite to link old and new revisions */
+#include "crc32.h"
 
 
+static size_t get_path_object_size(void* file_path_obj)
+{
+    assert(((C_struct_File_Path*)file_path_obj)->count > 0);
+
+    return sizeof(C_struct_File_Path) + ((C_struct_File_Path*)file_path_obj)->count;
+}
+
+
+void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
+{
+    C_struct_Path_And_File_Association old_assoc_obj;
+    C_struct_Path_And_File_Association new_assoc_obj;
+    C_struct_File old_file;
+    C_struct_File new_file;
+    uint8 kludge[FILE_BLOCK_SIZE];
+    C_struct_File_Path* ptr_path_obj = (C_struct_File_Path*)kludge;
+    ReferenceList* ref_list;
+    int num_refs;
+    int i;
+    ObjRef object_class;
+
+    nwos_read_object_from_disk(new_assoc_ref, &new_assoc_obj, sizeof(new_assoc_obj));
+
+    nwos_read_object_from_disk(&new_assoc_obj.file, &new_file, sizeof(new_file));
+
+    assert(is_void_reference(&new_file.header.object.prev_version));
+
+    nwos_read_variable_sized_object_from_disk(&new_assoc_obj.path, kludge, sizeof(kludge), &get_path_object_size);
+
+    ref_list = nwos_malloc_reference_list(&ptr_path_obj->header.object.references);
+
+    num_refs = ref_list->common_header.num_refs;
+
+    assert(is_same_object(&ref_list->references[num_refs-1], new_assoc_ref));
+
+    if (num_refs > 1)
+    {
+	for (i = 0; i < num_refs - 1; i++)
+	{
+	    nwos_get_object_class(&ref_list->references[i], &object_class);
+	    if (is_same_object(&object_class, &new_assoc_obj.header.common.class_definition))
+	    {
+		nwos_read_object_from_disk(&ref_list->references[i], &old_assoc_obj, sizeof(old_assoc_obj));
+
+		nwos_read_object_from_disk(&old_assoc_obj.file, &old_file, sizeof(old_file));
+
+		printf("Old association: %08x  file: %08x  next_version: %08x\n",
+		       nwos_ref_to_word(&ref_list->references[i]),
+		       nwos_ref_to_word(&old_assoc_obj.file),
+		       nwos_ref_to_word(&old_file.header.object.next_version));
+
+		if (is_void_reference(&old_file.header.object.next_version)) break;
+	    }
+	}
+
+	if (i < num_refs - 1)
+	{
+	    copy_reference(&old_file.header.object.next_version, &new_assoc_obj.file);
+	    copy_reference(&new_file.header.object.prev_version, &old_assoc_obj.file);
+
+	    nwos_crc32_calculate((uint8*) &old_file.header.object, sizeof(ObjectHeader), old_file.header.common.header_chksum);
+	    nwos_crc32_calculate((uint8*) &new_file.header.object, sizeof(ObjectHeader), new_file.header.common.header_chksum);
+
+	    nwos_overwrite_object_to_disk(&old_assoc_obj.file, &old_file, sizeof(old_file));
+	    nwos_overwrite_object_to_disk(&new_assoc_obj.file, &new_file, sizeof(new_file));
+	    printf("linked - old: %08x and new: %08x\n",
+		   nwos_ref_to_word(&old_assoc_obj.file),
+		   nwos_ref_to_word(&new_assoc_obj.file));
+	  }
+    }
+}
+
 
 int main(int argc, char* argv[])
 {
@@ -128,18 +210,25 @@ int main(int argc, char* argv[])
     ObjRef disc_list_ref;
     DIR* dp;
     struct dirent *dir_entry;
+    bool add_revision = false;
+    int argi = 1;
 
-
-    if (argc != 2)
+    if (argc == 3 && strcmp(argv[1], "--add-revision") == 0)
     {
-	fprintf(stderr, "usage: %s directory\n", argv[0]);
+	add_revision = true;
+	argi = 2;
+    }
+    else if (argc != 2)
+    {
+	fprintf(stderr, "usage: %s [--add-revision] directory\n", argv[0]);
+	fprintf(stderr, "  --add-revision  if file already exists with same name, make new file a new revision\n");
 	exit(1);
     }
 
-    dp = opendir(argv[1]);
+    dp = opendir(argv[argi]);
     if (dp == NULL)
     {
-	perror(argv[1]);
+	perror(argv[argi]);
 	exit(1);
     }
 
@@ -176,7 +265,7 @@ int main(int argc, char* argv[])
 
     if (closedir(dp) != 0)
     {
-	perror(argv[1]);
+	perror(argv[argi]);
 	exit(1);
     }
 
@@ -193,7 +282,7 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_initialize_objectify(bf_key, linear, serial, DEFAULT_TYPE_RW, DEFAULT_FILE);
+    nwos_initialize_objectify(bf_key, linear, serial, READ_WRITE, DEFAULT_FILE);
 
     nwos_set_root_object(&root_object_ref);
 
@@ -247,6 +336,7 @@ int main(int argc, char* argv[])
     if (nwos_find_disc_list(id, &ref))
     {
 	printf("That disc already exists!\n");
+	nwos_terminate_objectify();
 	exit(1);
     }
 
@@ -256,7 +346,7 @@ int main(int argc, char* argv[])
     {
 	start_time = time(NULL);
 
-	result = nwos_create_file_without_storing_data(argv[1], file_names[i], &files[i]);
+	result = nwos_create_file_without_storing_data(argv[argi], file_names[i], &files[i]);
 
 	assert(!is_void_reference(&files[i]));
 
@@ -265,6 +355,11 @@ int main(int argc, char* argv[])
 	    printf("created new file path: %02x%02x%02x%02x   time: %d seconds\n", 
 		   files[i].id[0], files[i].id[1], files[i].id[2], files[i].id[3], 
 		   (int) (time(NULL) - start_time));
+
+	    if (add_revision)   /* check to see if there was a previous revision of this file */
+	    {
+		check_for_previous_version_and_link(&files[i]);
+	    }
 	}
 	else
 	{
