@@ -27,6 +27,28 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.76  2008/02/25 14:31:20  jsedwards
+-- Changed nwos_allocate_new_chunk function to allow creating a new chunk for
+-- any reference and not just greater than the existing references.
+--
+-- Revision 1.75  2008/02/24 21:00:18  jsedwards
+-- Added code to allocate more chunks if the existing chunks are more than 90%
+-- full.
+--
+-- Revision 1.74  2008/02/24 20:30:44  jsedwards
+-- Fix Bug #1900855 - changed to just pick a block at random and if that fails
+-- to just do a linear search.
+--
+-- Revision 1.73  2008/02/11 04:33:37  jsedwards
+-- Changed to set 'read_only' flag for PUBLIC AccessType too.
+--
+-- Revision 1.72  2008/02/11 03:59:57  jsedwards
+-- Added "PUBLIC" AccessType for opening only the Public Objects.
+--
+-- Revision 1.71  2008/02/09 17:43:19  jsedwards
+-- Moved backup code out of "if (private..." so that backup files will be made
+-- in PUBLIC_MODE as well.
+--
 -- Revision 1.70  2008/02/06 03:25:58  jsedwards
 -- Added initializer for compressed_file boolean variable so compiler won't
 -- complain about it being uninitialized.
@@ -591,9 +613,9 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     /* make sure the storage is something we can deal with */
 #ifdef PUBLIC_MODE
-    assert(type == 0);
+    assert(type == PUBLIC);
 #else
-    assert(type == READ_ONLY || type == READ_WRITE);
+    assert(type == PUBLIC || type == READ_ONLY || type == READ_WRITE);
 #endif
 
     assert(nwos_private_path == NULL);   /* make sure this is only called once */
@@ -604,21 +626,24 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     nwos_public_path = nwos_get_public_objects_path();
 
-#ifdef PUBLIC_MODE
-    assert(path == NULL);
-#else
-    if (path == NULL)    /* no compressed file was passed in */
+    if (type == PUBLIC)
     {
-	nwos_private_path = nwos_get_private_objects_path();
-	compressed_file = (strstr(nwos_private_path, "compressed") != NULL);
+	assert(path == NULL);
     }
     else
     {
-	assert(type == READ_ONLY);
-	nwos_private_path = path;
-	compressed_file = true;
+	if (path == NULL)    /* no compressed file was passed in */
+	{
+	    nwos_private_path = nwos_get_private_objects_path();
+	    compressed_file = (strstr(nwos_private_path, "compressed") != NULL);
+	}
+	else
+	{
+	    assert(type == READ_ONLY);
+	    nwos_private_path = path;
+	    compressed_file = true;
+	}
     }
-#endif
 
     /********************************/
     /* open the public objects file */
@@ -671,7 +696,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     if (nwos_private_path != NULL)
     {
-	read_only = (type == READ_ONLY);
+	read_only = (type == PUBLIC || type == READ_ONLY);
 
 	if (read_only)
 	{
@@ -830,18 +855,18 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	snprintf(log_msg, sizeof(log_msg), "   private blocks: %9u  used: %9u", nwos_total_private_blocks, nwos_used_private_blocks);
 	nwos_log(log_msg);
+    }
 
 #ifndef NO_BACKUP
-	/* Do a backup file? */
+    /* Do a backup file? */
 
-	backup_dir = nwos_get_backup_directory_path();
+    backup_dir = nwos_get_backup_directory_path();
 
-	if (backup_dir != NULL && !read_only)
-	{
-	    nwos_initialize_backup(backup_dir, &disk_header);
-	}
-#endif
+    if (backup_dir != NULL && !read_only)
+    {
+	nwos_initialize_backup(backup_dir, &disk_header);
     }
+#endif
 }
 
 
@@ -979,39 +1004,42 @@ void nwos_allocate_new_chunk(uint32 ref)
 {
     uint32 offset;
     static uint8* chunk;
-    float ratio;          /* possible chunks to actual chunks on disk ratio */
-    uint32 index;
+    int i;
+
+    assert(nwos_used_private_chunks < nwos_total_private_chunks);
 
     assert(RESERVED_PUBLIC_BLOCKS <= ref && ref <= MAXIMUM_VALID_PRIVATE_REFERENCE);
 
     assert(nwos_hash_uint32_ref(ref) == 0);
 
-    assert(nwos_used_private_chunks == 0 || ref >= chunk_info[nwos_used_private_chunks-1].ref + USABLE_BLOCKS_PER_CHUNK);
+    /* figure out where this ref goes in the chunk_info table */
+    for (i = nwos_used_private_chunks; i > 0; i--)
+    {
+	if (chunk_info[i].ref < ref) break;
+
+	chunk_info[i].ref = chunk_info[i-1].ref;
+	chunk_info[i].ref = chunk_info[i-1].used;
+	chunk_info[i].ref = chunk_info[i-1].index;
+    }
 
     offset = (ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
 
-    chunk_info[nwos_used_private_chunks].ref = ref - offset;
+    chunk_info[i].ref = ref - offset;
 
-
-    offset = chunk_info[nwos_used_private_chunks].ref - RESERVED_PUBLIC_BLOCKS;
+    assert(i == 0 || chunk_info[i-1].ref < chunk_info[i].ref);
+    assert(i == nwos_used_private_chunks || chunk_info[i+1].ref > chunk_info[i].ref);
 
     /* make sure chunk_info.ref is an even multiple of USABLE_BLOCKS_PER_CHUNK */
 
+    offset = chunk_info[i].ref - RESERVED_PUBLIC_BLOCKS;
+
+    chunk_info[i].used = 0;
+    chunk_info[i].index = nwos_used_private_chunks;
+
+    assert(chunk_info[i].index < nwos_total_private_chunks);
+
     assert((offset % USABLE_BLOCKS_PER_CHUNK) == 0);
 
-    ratio = (float) nwos_total_private_chunks / (float) RESERVED_PRIVATE_CHUNKS;
-
-    index = (uint32) ((offset / USABLE_BLOCKS_PER_CHUNK) * ratio + 0.5);
-
-    assert(index < RESERVED_PRIVATE_CHUNKS);
-
-    if (nwos_used_private_chunks > 0 && index <= chunk_info[nwos_used_private_chunks-1].index)  /* already used */
-    {
-	index = chunk_info[nwos_used_private_chunks-1].index + 1;
-    }
-
-    chunk_info[nwos_used_private_chunks].index = index;
-
     if (chunk == NULL)   /* first time this has been called, allocate memory */
     {
 	chunk = malloc(CHUNK_SIZE);
@@ -1030,7 +1058,7 @@ void nwos_allocate_new_chunk(uint32 ref)
 	chunk[3] = 0xff;
     }
 
-    offset = nwos_block_offset_to_chunks + (index * BLOCKS_IN_CHUNK);
+    offset = nwos_block_offset_to_chunks + (chunk_info[i].index * BLOCKS_IN_CHUNK);
 
     if (lseek(private_file_desc, (off_t)offset << 8, SEEK_SET) < 0)
     {
@@ -1654,7 +1682,6 @@ static void generate_new_completely_random_id(ObjRef* ref)
     int byte;
     int chunk = 0;
     Bit_Map_Cache_Entry* entry;
-    uint16 quota;
     uint32 word;
 #ifdef LOG_ID_GENERATION
     char msg[128];
@@ -1666,19 +1693,28 @@ static void generate_new_completely_random_id(ObjRef* ref)
     {
 #define UBPC USABLE_BLOCKS_PER_CHUNK
 
-	for (quota = UBPC / 4; quota <= UBPC; quota += UBPC / 16)
+	/* first try to find one with some empties at random */
+	for (i = 0; i < nwos_used_private_chunks; i++)
 	{
 	    chunk = random() % nwos_used_private_chunks;
 
-	    if (chunk_info[chunk].used < quota) break;
+	    if (chunk_info[chunk].used < USABLE_BLOCKS_PER_CHUNK) break;
+	}
+
+	if (i == nwos_used_private_chunks)   /* didn't find one, just do a search */
+	{
+	    for (chunk = 0; chunk < nwos_used_private_chunks; chunk++)
+	    {
+		if (chunk_info[chunk].used < USABLE_BLOCKS_PER_CHUNK) break;
+	    }
 	}
     }
 
+    assert(chunk_info[chunk].used < USABLE_BLOCKS_PER_CHUNK);
+
     entry = find_bit_map_in_cache(nwos_block_offset_to_chunks +
 				 ((uint32)chunk_info[chunk].index * BLOCKS_IN_CHUNK) + BIT_MAP_BLOCKS);
 
-    assert(chunk_info[chunk].used < USABLE_BLOCKS_PER_CHUNK);
-
     /* don't pick one of the bytes (currently 0-3) used by the bit map */
     byte = (random() % (BIT_MAP_BYTES - (BIT_MAP_BLOCKS / 8))) + (BIT_MAP_BLOCKS / 8);
 
@@ -1873,6 +1909,7 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
     uint32 max;
     uint32 ref;
     char msg[80];
+    bool need_to_allocate_more_chunks;
 
     assert(estimated_blocks > 0);
 
@@ -1913,7 +1950,19 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
 
     allocated_blocks = nwos_used_private_chunks * USABLE_BLOCKS_PER_CHUNK;
 
-    while (allocated_blocks <= nwos_used_private_blocks + block_estimate)
+    /* if more than 90% of the allocated blocks are used allocate some more */
+    if (nwos_total_private_chunks > nwos_used_private_chunks && 
+	nwos_used_private_blocks > (nwos_used_private_chunks * USABLE_BLOCKS_PER_CHUNK / 10) * 9)
+    {
+	need_to_allocate_more_chunks = true;
+	nwos_log("NOTE: more than 90%% full, allocating more chunks");
+    }
+    else
+    {
+	need_to_allocate_more_chunks = allocated_blocks <= nwos_used_private_blocks + block_estimate;
+    }
+
+    while (need_to_allocate_more_chunks)
     {
 	/* allocate can currently only deal with adding chunks at the end */
 	if (chunk_info[nwos_used_private_chunks-1].index + 1 >= nwos_total_private_chunks)
@@ -1949,6 +1998,8 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
 	assert(nwos_used_private_chunks * USABLE_BLOCKS_PER_CHUNK > allocated_blocks);
 
 	allocated_blocks = nwos_used_private_chunks * USABLE_BLOCKS_PER_CHUNK;
+
+	need_to_allocate_more_chunks = allocated_blocks <= nwos_used_private_blocks + block_estimate;
     }
 
     return true;
