@@ -27,6 +27,14 @@
 --
 --
 -- $Log: list_files.c,v $
+-- Revision 1.24  2008/02/16 17:34:34  jsedwards
+-- Added if statements in print_sha1 to print "not available" if it is void in
+-- the file object and print_time to print "unknown" if modification time is
+-- zero.
+--
+-- Revision 1.23  2008/02/11 04:15:12  jsedwards
+-- Added the --public option.
+--
 -- Revision 1.22  2008/02/03 01:05:23  jsedwards
 -- Changed DEFAULT_TYPE_RO to READ_ONLY.
 --
@@ -120,7 +128,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "objectify.h"
+#include "objectify_private.h"   /* do private so that nwos_get_public_objects_path is defined */
 #include "time_stamp.h"
 
 
@@ -153,10 +161,21 @@ void print_sha1(ObjRef* file_ref)
     int j;
 
     nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj));  /* read the file object */
-    nwos_read_object_from_disk(&file_obj.sha1sum, &sha1_object, sizeof(sha1_object));
 
-    for (j = 0; j < 20; j++) printf("%02x", sha1_object.sha1sum[j]);
-    printf("  ");
+    if (is_void_reference(&file_obj.sha1sum))
+    {
+             /*          1         2         3         4 */
+             /* 1234567890123456789012345678901234567890 */
+
+	printf("* No SHA1 checksum stored for this file * ");
+    }
+    else
+    {
+	nwos_read_object_from_disk(&file_obj.sha1sum, &sha1_object, sizeof(sha1_object));
+
+	for (j = 0; j < 20; j++) printf("%02x", sha1_object.sha1sum[j]);
+	printf("  ");
+    }
 }
 
 
@@ -188,13 +207,20 @@ void print_sha256(ObjRef* file_ref)
 
 void print_time(TimeStamp ts)
 {
-    printf("  %u-%02u-%02u %02u:%02u:%02u",
-	   nwos_extract_year_from_time_stamp(ts),
-	   nwos_extract_month_from_time_stamp(ts),
-	   nwos_extract_day_of_month_from_time_stamp(ts),
-	   nwos_extract_hour_from_time_stamp(ts),
-	   nwos_extract_minute_from_time_stamp(ts),
-	   nwos_extract_second_from_time_stamp(ts));
+    if (nwos_time_stamp_is_zero(ts))
+    {
+	printf("  unknown");
+    }
+    else
+    {
+	printf("  %u-%02u-%02u %02u:%02u:%02u",
+	       nwos_extract_year_from_time_stamp(ts),
+	       nwos_extract_month_from_time_stamp(ts),
+	       nwos_extract_day_of_month_from_time_stamp(ts),
+	       nwos_extract_hour_from_time_stamp(ts),
+	       nwos_extract_minute_from_time_stamp(ts),
+	       nwos_extract_second_from_time_stamp(ts));
+    }
 }
 
 
@@ -228,6 +254,7 @@ int main(int argc, char* argv[])
     bool do_list_old_names = false;
     bool do_time = false;
     bool do_revision = false;
+    bool do_public = false;
     bool match;
 
 
@@ -248,6 +275,11 @@ int main(int argc, char* argv[])
 	    do_md5 = true;
 	    argn++;
 	}
+	else if (strcmp(argv[argn], "--public") == 0)
+	{
+	    do_public = true;
+	    argn++;
+	}
 	else if (strcmp(argv[argn], "--sha1") == 0)
 	{
 	    do_sha1 = true;
@@ -275,13 +307,15 @@ int main(int argc, char* argv[])
 	}
 	else
 	{
-	    fprintf(stderr, "usage: %s [--compressed compressed-file] [--time] [--md5 | --sha1 | --sha256] [--list-old-names] [--revision] [FILES]...\n", argv[0]);
+	    fprintf(stderr, "usage: %s [--public] [--compressed compressed-file] [--time] [--md5 | --sha1 | --sha256] [--list-old-names] [--revision] [FILES]...\n", argv[0]);
 	    fprintf(stderr, "\n");
 	    fprintf(stderr, "List information about the files stored or logged in Objectify.\n");
 	    fprintf(stderr, "If no files are specified, all files stored and logged are printed.\n");
 	    fprintf(stderr, "\n");
+	    fprintf(stderr, "  --compresed file  read from 'file' instead of normal storage\n");
 	    fprintf(stderr, "  --list-old-names  print old names of files (that have been renamed)\n");
 	    fprintf(stderr, "  --md5             print md5 sum of each file\n");
+	    fprintf(stderr, "  --public          list public files instead of private\n");
 	    fprintf(stderr, "  --revision        print revision of each file\n");
 	    fprintf(stderr, "  --sha1            print sha1 sum of each file\n");
 	    fprintf(stderr, "  --sha256          print sha256 sum of each file\n");
@@ -293,24 +327,49 @@ int main(int argc, char* argv[])
 
     }
 
+    if (do_public && path != NULL)   /* both --public and --compressed were specified */
+    {
+	fprintf(stderr, "WARNING: because --public was specified --compressed file is ignored!\n");
+	path = NULL;
+    }
+
     printf("\n");
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
-
-    nwos_set_root_object(&root_object_ref);
-
-
-    if (!nwos_find_private_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref))
+    if (do_public)
     {
-	printf("No files are stored in the system!\n");
-	exit(1);
+	nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
+
+	root_object_ref.id[0] = 0;
+	root_object_ref.id[1] = 0;
+	root_object_ref.id[2] = 0;
+	root_object_ref.id[3] = 1;
+
+	nwos_set_root_object(&root_object_ref);
+
+	if (!nwos_find_public_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref))
+	{
+	    printf("No public files found, please update your Public Objects file: %s!\n", nwos_get_public_objects_path());
+	    exit(1);
+	}
+    }
+    else
+    {
+	nwos_get_key_from_password(big_key, sizeof(big_key));
+
+	memcpy(bf_key, big_key, 16);
+	linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
+	memcpy(root_object_ref.id, big_key+20, 4);
+	serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
+
+	nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
+
+	nwos_set_root_object(&root_object_ref);
+
+	if (!nwos_find_private_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref))
+	{
+	    printf("No files are stored in the system!\n");
+	    exit(1);
+	}
     }
 
     nwos_read_class_definition(&assoc_class_ref, &class_def_obj);
