@@ -27,6 +27,14 @@
 --
 --
 -- $Log: log.c,v $
+-- Revision 1.14  2008/03/12 14:43:59  jsedwards
+-- Added code to create subdirectories in the path if they don't already
+-- exist.
+--
+-- Revision 1.13  2008/03/12 11:40:48  jsedwards
+-- Added code to disable logging if nwos_log_arguments is called with argc=0
+-- and argv=NULL.
+--
 -- Revision 1.12  2008/02/05 04:23:44  jsedwards
 -- Added include of objectify_private.h.  Removed forward references for
 -- strlcat and strlcpy because they are declared in objectify.h.  Changed to
@@ -75,9 +83,11 @@
 
 
 #include <assert.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 #include <sys/time.h>
 #include <time.h>
 #include <unistd.h>
@@ -86,6 +96,8 @@
 #include "objectify_private.h"  /* include this to get the nwos_get_log_file_path function */
 
 
+static bool log_disabled = false;
+
 
 /******************************************/
 /* Log file - THIS NEEDS TO GO AWAY SOON. */
@@ -136,6 +148,7 @@ void append_date_from_time_string(char* dst, char* time_str, size_t length)
     dst[i++] = '\0';
 }
 
+
 void nwos_log(char* str)
 {
     struct timeval tv;
@@ -144,6 +157,9 @@ void nwos_log(char* str)
     static char* log_file_path;
     size_t path_len;
     const char* path;
+    char* p;
+
+    if (log_disabled) return;    /* if logging has been disabled, just return */
 
     gettimeofday(&tv, NULL);
     strncpy(time_str, ctime((time_t*)&tv.tv_sec), 32);
@@ -152,6 +168,27 @@ void nwos_log(char* str)
     if (log_file_path == NULL)
     {
 	path = nwos_get_log_file_path();
+
+	/* NOTE: this is a bad thing to do, it temporarily modifies */
+	/* the string returned by nwos_get_log_file_path.           */
+        /* We cheat the system by calling strchr.                   */
+
+	p = strchr(path, '/');
+	assert(p != NULL);
+	p = strchr(p + 1, '/');
+	while (p != NULL)
+	{
+	    *p = '\0';
+	    if (mkdir(path, 0755) != 0 && errno != EEXIST)
+	    {
+		perror(p);
+		exit(1);
+	    }
+	    *p = '/';
+
+	    p = strchr(p + 1, '/');
+	}
+
 	path_len = strlen(path) + 1 + 8 + 4 + 1;
 
 	log_file_path = malloc(path_len);
@@ -183,6 +220,16 @@ void nwos_log_arguments(int argc, char* argv[])
     char* src;
     char msg[16384] = "Program:";
 
+    if (argc == 0 || argv == NULL)
+    {
+	assert(argc == 0 && argv == NULL);    /* if one is zero, they should both be */
+	log_disabled = true;
+	return;
+    }
+
+    assert(argc > 0);
+    assert(argv != NULL);
+
     i = strlen(msg);
 
     assert(msg[i] == '\0');
