@@ -27,6 +27,19 @@
 --
 --
 -- $Log: objectify.c,v $
+-- Revision 1.145  2008/02/27 05:48:32  jsedwards
+-- Change so that if no blowfish, linear, and serial values are specified
+-- (as is the case for expand_sparse) it doesn't try to initialize the
+-- security stuff.
+--
+-- Revision 1.144  2008/02/15 15:23:57  jsedwards
+-- Fix to compile correctly in PUBLIC_MODE without blowfish_key,
+-- linear_after_first, and serial_after_first.  They were wrapped in
+-- #ifndef PUBLIC_MODE to eliminate warnings in revision 1.141.
+--
+-- Revision 1.143  2008/02/11 04:09:40  jsedwards
+-- Changed to handle new PUBLIC AccessType to only open Public Objects.
+--
 -- Revision 1.142  2008/02/03 02:14:42  jsedwards
 -- Change parameter to initialize_objectify to AccessType instead of StorgeType
 -- and add 'const' to path parameter.
@@ -988,7 +1001,9 @@ static void generate_sequence_table(uint32* linear, uint32* serial, uint8 table[
 
 void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear_seed, uint32 serial_seed, AccessType type, const char* path)
 {
-#ifndef PUBLIC_MODE
+#ifdef PUBLIC_MODE
+    assert(type == PUBLIC);
+#else
     int i;
     int j;
     int k;
@@ -1002,52 +1017,54 @@ void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear_seed, uint32 seri
 
     /* are we starting up with security or not? */
 
-    if (bf_key == NULL)    /* this is kind of cheesy since bf_key is declared as an array */
+    if (type == PUBLIC)
     {
-	assert(linear_seed == 0 && serial_seed == 0);
+	assert(bf_key == NULL && linear_seed == 0 && serial_seed == 0);
     }
+#ifndef PUBLIC_MODE
     else
     {
-#if 0
-	int i;
-	printf("blowfish key: ");
-	for (i = 0; i < 16; i++) printf("%02x", bf_key[i]);
-	printf("\n");
-#endif
-
-#ifdef PUBLIC_MODE
-	bool Initialize_Called_With_Non_Null_Key = false;
-	assert(Initialize_Called_With_Non_Null_Key);
-#else
-	BF_set_key(&blowfish_key, 16, bf_key);
-
-	linear_after_first = linear_seed;
-	serial_after_first = serial_seed;
-
-	/* Create all of the sequence tables from the linear and serial parameters */
-	for (i = 0; i < NUM_STORED_SEQ; i++)
+	if (bf_key != NULL)
 	{
-	    tries = 0;
-
-	    do
-	    {
-		tries++;
-
-		assert(tries < 4);
-
-		generate_sequence_table(&linear_after_first, &serial_after_first, nwos_random_sequence[i]);
-
-		for (j = 0; j < i; j++)
-		{
-		    for (k = 8; k < FILE_BLOCK_SIZE; k++) if (nwos_random_sequence[i][k] != nwos_random_sequence[j][k]) break;
-
-		    if (k == FILE_BLOCK_SIZE) break;
-		}
-	    }
-	    while (j < i);    /* if j is less than i it means one of the tables matched and needs to be regenerated */
-	}
+	    linear_after_first = linear_seed;
+	    serial_after_first = serial_seed;
+#if 0
+	    int i;
+	    printf("blowfish key: ");
+	    for (i = 0; i < 16; i++) printf("%02x", bf_key[i]);
+	    printf("\n");
 #endif
+	    BF_set_key(&blowfish_key, 16, bf_key);
+
+	    /* Create all of the sequence tables from the linear and serial parameters */
+	    for (i = 0; i < NUM_STORED_SEQ; i++)
+	    {
+		tries = 0;
+
+		do
+		{
+		    tries++;
+
+		    assert(tries < 4);
+
+		    generate_sequence_table(&linear_after_first, &serial_after_first, nwos_random_sequence[i]);
+
+		    for (j = 0; j < i; j++)
+		    {
+			for (k = 8; k < FILE_BLOCK_SIZE; k++) if (nwos_random_sequence[i][k] != nwos_random_sequence[j][k]) break;
+
+			if (k == FILE_BLOCK_SIZE) break;
+		    }
+		}
+		while (j < i);   /* if j is less than i it means one of the tables matched and needs to be regenerated */
+	    }
+	}
+	else   /* if the blowfish key is null, all of the values should be null */
+	{
+	    assert(bf_key == NULL && linear_seed == 0 && serial_seed == 0);
+	}
     }
+#endif
 }
 
 
@@ -1639,7 +1656,7 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 #ifndef USE_PREDEFINED_STRUCTS
 #ifndef PUBLIC_MODE
     /* check to see if this is a file object (which has an older revision) */
-    if (is_same_object(&header->common.class_definition, get_file_001_reference()))
+    if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, get_file_001_reference()))
     {
 	check_object_data(ref, (uint8*)object, sizeof(C_struct_File_001), header->common.data_chksum);
 	nwos_update_file_001_object_to_current(object, size);
@@ -1968,7 +1985,7 @@ void nwos_get_object_class(ObjRef* obj, ObjRef* object_class)
     }
 
 #ifndef USE_PREDEFINED_STRUCTS
-    if (is_same_object(&header.common.class_definition, get_file_001_reference()))
+    if (nwos_reference_type(obj) == Private_Reference && is_same_object(&header.common.class_definition, get_file_001_reference()))
     {
 	copy_reference(object_class, get_file_class_reference());
     }
