@@ -27,6 +27,16 @@
 --
 --
 -- $Log: objectify.h,v $
+-- Revision 1.131  2008/02/16 17:25:09  jsedwards
+-- Added external declaration for new nwos_add_info_to_existing_file function
+-- in file.c.
+--
+-- Revision 1.130  2008/02/16 13:17:25  jsedwards
+-- Added external declaration for new nwos_create_file_with_only_md5 function.
+--
+-- Revision 1.129  2008/02/11 04:03:42  jsedwards
+-- Added "PUBLIC" to AccessType.
+--
 -- Revision 1.128  2008/02/05 04:17:19  jsedwards
 -- Added 'const' to the source string parameter in the strlcat and strlcpy
 -- function declarations.
@@ -665,6 +675,7 @@
 #define DEFAULT_FILE NULL      /* this used to define a path, now a null tells disk_io to use the default */
 
 typedef enum {
+  PUBLIC     = 0x50424C43,
   READ_ONLY  = 0x52454144,
   READ_WRITE = 0x52445752,
 } AccessType;
@@ -1022,6 +1033,8 @@ extern bool nwos_find_matching_path_and_file_association(char* path, char* file,
 extern CheckFileResult nwos_check_file_checksums(char* path);
 extern bool nwos_file_is_stored(ObjRef* association_ref);  /* returns true if data stored in system */
 extern bool nwos_file_has_backup(ObjRef* association_ref);  /* returns true if file is on a backup disc */
+extern ObjCreateResult nwos_create_file_with_only_md5(char* file_name, uint8 md5_digest[MD5_DIGEST_SIZE], ObjRef* ref);
+extern ObjCreateResult nwos_add_info_to_existing_file(char* file_name, ObjRef* ref);
 extern ObjCreateResult nwos_create_file_without_storing_data(char* path, char* file_name, ObjRef* ref);
 extern ObjCreateResult nwos_create_file(char* path, ObjRef* ref);
 extern ObjCreateResult nwos_add_new_revision_of_file(char* path, ObjRef* ref);
