@@ -27,6 +27,10 @@
 --
 --
 -- $Log: rename_file.c,v $
+-- Revision 1.15  2008/02/27 05:46:14  jsedwards
+-- Changed so that either --keep-history or --no-history must be specified,
+-- it will not default to either one.
+--
 -- Revision 1.14  2008/02/03 01:00:55  jsedwards
 -- Changed DEFAULT_TYPE_RW to READ_WRITE.
 --
@@ -223,24 +227,24 @@ int main(int argc, char* argv[])
     char *id;
     char* old_name = "-";
     char* new_name = "-";
-    bool no_history = false;
+    bool keep_history = false;
 
 
-    if (argc == 3)
+    if (argc == 4 && strcmp(argv[1], "--keep-history") == 0)
     {
-	old_name = argv[1];
-	new_name = argv[2];
+	keep_history = true;
+	old_name = argv[2];
+	new_name = argv[3];
     }
     else if (argc == 4 && strcmp(argv[1], "--no-history") == 0)
     {
-	no_history = true;
 	old_name = argv[2];
 	new_name = argv[3];
     }
 
     if (*old_name == '-' || *new_name == '-')
     {
-	fprintf(stderr, "usage: %s [--no-history] old-file-name new-file-name\n", argv[0]);
+	fprintf(stderr, "usage: %s --no-history | --keep-history old-file-name new-file-name\n", argv[0]);
 	exit(1);
     }
 
@@ -269,7 +273,7 @@ int main(int argc, char* argv[])
 
     nwos_set_security_level(Security_Very_Low);
 
-    if (!no_history && !nwos_check_blocks_available(4))
+    if (keep_history && !nwos_check_blocks_available(4))
     {
 	fprintf(stderr, "Cannot rename with history (could rename with --no-history option).\n");
 	nwos_terminate_objectify();
@@ -338,7 +342,11 @@ int main(int argc, char* argv[])
     assert(is_same_object(&ptr_path_obj->header.common.id, &old_path_ref));
     assert(is_same_object(&assoc_obj.header.common.id, &old_assoc_ref));
 
-    if (no_history)
+    if (keep_history)
+    {
+	create_new_path(new_name, &assoc_obj);   /* assoc_obj is modified by this function */
+    }
+    else
     {
 	ptr_path_obj->count = length;
 
@@ -348,10 +356,6 @@ int main(int argc, char* argv[])
 
 	nwos_overwrite_object_to_disk(&old_path_ref, ptr_path_obj, get_path_object_size(ptr_path_obj));
     }
-    else
-    {
-	create_new_path(new_name, &assoc_obj);   /* assoc_obj is modified by this function */
-    }
 
     nwos_terminate_objectify();
 
