@@ -27,6 +27,14 @@
 --
 --
 -- $Log: user_config.c,v $
+-- Revision 1.12  2008/02/24 13:24:33  jsedwards
+-- Fix so that if DEFAULT_USER_CONFIG_PATH contains a leading '~' the user's
+-- home directory will correctly be substituted.
+--
+-- Revision 1.11  2008/02/23 02:06:26  jsedwards
+-- Changed to search for a configuration file in the users home directory
+-- first and then read the system configuration file if it doesn't exist.
+--
 -- Revision 1.10  2008/02/06 04:19:27  jsedwards
 -- Fix Bug #1887042 - changed so that all environment variables are checked
 -- before reading the config file.
@@ -65,6 +73,7 @@
 */
 
 #include <ctype.h>
+#include <limits.h>    /* define PATH_MAX */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -91,14 +100,43 @@ static struct path_table
 #define PATH_TABLE_SIZE (sizeof(path_table) / sizeof(struct path_table))
 
 
+/* get the user's home directory */
+
+const char* get_users_home_dir(const char* path)
+{
+    static char* result = NULL;
+    char msg[128];
+
+    if (result == NULL)
+    {
+	result = getenv("HOME");
+
+	if (result == NULL)
+	{
+	    snprintf(msg, sizeof(msg), "WARNING: $HOME environment variable not defined, cannot resolve path: %s", path);
+
+	    nwos_log(msg);
+	    fprintf(stderr, "%s\n", msg);
+	}
+    }
+
+    return result;
+}
+
+
 static void read_configuration_file()
 {
     FILE* fp;
+    char config_path[PATH_MAX];
     char buffer[1024];
     int line_number = 0;
     char *p;
     int i;
 
+    assert(DEFAULT_SYSTEM_CONFIG_PATH[0] == '/');
+
+    assert(DEFAULT_USER_CONFIG_PATH[0] == '/' || (DEFAULT_USER_CONFIG_PATH[0] == '~' && DEFAULT_USER_CONFIG_PATH[1] == '/'));
+
     if (!config_file_has_been_read)
     {
 	config_file_has_been_read = true;
@@ -112,7 +150,25 @@ static void read_configuration_file()
 
 	/* then read the file */
 
-	fp = fopen(DEFAULT_CONFIG_PATH, "r");
+	/* first try to open a users config file */
+	if (*DEFAULT_USER_CONFIG_PATH == '~')
+	{
+	    strlcpy(config_path, get_users_home_dir(DEFAULT_USER_CONFIG_PATH), sizeof(config_path));
+	    strlcat(config_path, DEFAULT_USER_CONFIG_PATH+1, sizeof(config_path));
+	}
+	else
+	{
+	    strlcpy(config_path, DEFAULT_USER_CONFIG_PATH, sizeof(config_path));
+	}
+
+	fp = fopen(config_path, "r");
+
+	/* if that failed try to open the system config file */
+	if (fp == NULL)
+	{
+	    strlcpy(config_path, DEFAULT_SYSTEM_CONFIG_PATH, sizeof(config_path));
+	    fp = fopen(config_path, "r");
+	}
 
 	if (fp != NULL)
 	{
@@ -127,7 +183,7 @@ static void read_configuration_file()
 		    if (buffer[0] != '#')
 		    {
 			snprintf(buffer, sizeof(buffer), "WARNING: config file '%s' line %d is too long, ignored",
-				 DEFAULT_CONFIG_PATH, line_number);
+				 config_path, line_number);
 
 			nwos_log(buffer);
 			fprintf(stderr, "%s\n", buffer);
@@ -174,7 +230,7 @@ static void read_configuration_file()
 			    else
 			    {
 				snprintf(buffer, sizeof(buffer), "WARNING: config file '%s' line %d is invalid, ignored",
-					 DEFAULT_CONFIG_PATH, line_number);
+					 config_path, line_number);
 
 				nwos_log(buffer);
 				fprintf(stderr, "%s\n", buffer);
@@ -205,10 +261,10 @@ static void read_configuration_file()
 static const char* get_path(char* name)
 {
     int i;
-    char* home;
+    const char* home;
     char* combined;
     size_t combined_length;
-    char msg[128];
+
 
     for (i = 0; i < PATH_TABLE_SIZE; i++)
     {
@@ -224,16 +280,10 @@ static const char* get_path(char* name)
 
     if (path_table[i].path != NULL && path_table[i].path[0] == '~')
     {
-	home = getenv("HOME");
+	home = get_users_home_dir(path_table[i].path);
 
 	if (home == NULL)
 	{
-	    snprintf(msg, sizeof(msg), "WARNING: $HOME environment variable not defined, cannot resolve path: %s",
-		     path_table[i].path);
-
-	    nwos_log(msg);
-	    fprintf(stderr, "%s\n", msg);
-
 	    if (path_table[i].path_length != 0)  /* path was malloc'd */
 	    {
 		free(path_table[i].path);
